function[output,input] = dfRemoval2(input)
% function[output,input] = dfRemoval2(input)
% takes an input signal (which should be a single analysis frame) and
% removes intra-frame frequency change from invidual components
% NB this code is for illustrative purposes only and so contains only
% minimal error checking
% N.B. '2' is a more complicated version which actually recalculates the
% entire window
% Jez Wells, University of York, 2010.

zeroPad = 8;
Fs = 44100;

if(min(size(input))>1)
    error('function only works with single channel input');
end
N = length(input);
if(N~=1025)
    warning('function designed for 1025 length input')
end

[standardFFT,timeFFT,freqFFT,FFTSize,input] = ReassignmentAnalysis(input,zeroPad);
complexSize = round(FFTSize*0.5)+1;
binWidth = Fs/FFTSize;

% find component regions
[lower,upper,maxima] = regionFind(abs(standardFFT),zeroPad);

% now deal with each component region in turn
for n = 1:length(lower)
    % estimate parameters
    [fEst(n),aEst(n),dfEst(n),daEst(n),phi(n)] = sineParameters(input,standardFFT,timeFFT,freqFFT,maxima(n),FFTSize);
    % synthesize new spectrum
    % first find centre frequencies of bins
    binFreqs = [lower(n)-1:upper(n)-1]*binWidth; % -1 as Matlab indices start at 1
    binFreqs = binFreqs-fEst(n); % exact position of component = 0, so that window is symmetric around this point
    binFreqs = binFreqs*(N/Fs);
    % convert da estimate in dB to estimate in Nepers
    Nep = daEst(n)*log(10)/20;
    temp = j*2*pi*binFreqs;
    numerator = 4*(pi^2)*sinh(.5*(Nep-temp));  
    denominator = (Nep-temp).*((Nep^2)-(2*temp*Nep)-((4*(pi^2)).*((binFreqs.^2)-1)));
    energy1 = sum(abs(standardFFT(lower(n):upper(n))).^2);
    standardFFT(lower(n):upper(n)) = numerator./denominator;  
    energy2 = sum(abs(standardFFT(lower(n):upper(n))).^2);
    standardFFT(lower(n):upper(n)) = standardFFT(lower(n):upper(n))*(sqrt(energy1/energy2));
    % now convert to polar coordinates and adjust the phase
    [ph,mag] = cart2pol(real(standardFFT(lower(n):upper(n))),imag(standardFFT(lower(n):upper(n))));
    ph = ph+phi(n);
    % convert back
    [rl,im] = pol2cart(ph,mag);
    standardFFT(lower(n):upper(n)) = complex(rl,im);
end

% recombine modfied magnitude and phase
%rl = mag.*cos(ph);
im = mag.*sin(ph);
OUTPUT = zeros(1,FFTSize);
OUTPUT(1:complexSize) = standardFFT;
output = ifft(OUTPUT, 'symmetric');   
% undo zero-phase windowing
output = circshift(output,[0,-ceil(N*0.5)]);
input = circshift(input',[0,-ceil(N*0.5)]);





function[fEst,aEst,dfEst,daEst,phi] = sineParameters(frame,standardFFT,timeFFT,freqFFT,idx,FFTSize)
% This sub function uses iterative 2D reassignment distortion analysis to
% determine the sinusoidal parameters for the peak at idx

% performs 4 iterations, takes mean of estimates from last two iterations
iterations = 4;

frameSize = 1025;	%frame size
zeroPad = 8;	%zero-pad amount
range = 1;

daMult = 99/96;     % needed for faster DIY linear 2D interpolation
dfMult = 99/260;

en = 0.00390625 * 0.99863363968655;    %amplitude scaler for 1025 point Hann window

mag = abs(standardFFT);
binwidth = 44100/FFTSize;
recipBinwidth = 1/binwidth;
%derive frequencies
fVector = ReassignedFreqVector(standardFFT,freqFFT,FFTSize);

% get phase reassignment offsets
% perform complex division and multiply imaginary part by FFTsize/2Piin
phaseOffsets = real(timeFFT./standardFFT);

load tables %load up all table data (generated using lookupTableGen)

xA = daEst;
yA = daX;            % swap around as we are finding x from y
clear daEst daX;

xF = dfEst;
yF = dfX;            % swap around as we are finding x from y
clear dfEst dfX;

% load amp correction table 
load ampCorrect_260_96_1025_8x
% load amp correction coefficients
load ampCorrect_quintic_1025_8x

phi = angle(standardFFT(idx));
fEst = fVector(idx);
% estimate frequency and amplitude change
[dfMeasure,daMeasure] = Wells(standardFFT,freqFFT,timeFFT, FFTSize,fVector,idx,zeroPad,range);
dfSign = sign(dfMeasure);
daSign = sign(daMeasure);
dfMeasure = abs(dfMeasure);
daMeasure = abs(daMeasure);
% find maximum
xMax = max(xA(:,1));
xMin = min(xA(:,1));
if (xMax < daMeasure)
   daMeasure = xMax;
end
if (daMeasure < xMin)
   daMeasure = xMin;
end
daEst(1) = interp1DLin(xA(:,1),yA,daMeasure); % faster alternative to interp1

if(daEst(1) < 0.0)
  daEst(1) = 0.0;
end
if(daEst(1) > 96.0)
     daEst(1) = 96.0;
end

daPointerMult = 99/96;                        
daPointer = 1 + (daEst(1) * daPointerMult);
if (daPointer < 1)
   daPointer = 1;
end
if (daPointer > 100)
   daPointer = 100;
end
% find maximum
xMax = max(xF(ceil(daPointer),:));
xMin = min(xF(ceil(daPointer),:));
temp = dfMeasure;
if (xMax < temp)
    temp = xMax;
end
if (temp < xMin)
     temp = xMin;
end
hi = interp1DLin(xF(ceil(daPointer),:),yF,temp); % faster alternative to interp1
xMax = max(xF(floor(daPointer),:));
xMin = min(xF(floor(daPointer),:));
temp = dfMeasure;
if (xMax < temp)
     temp = xMax;
end
if (temp < xMin)
   temp = xMin;
end
lo = interp1DLin(xF(floor(daPointer),:),yF,temp); % faster alternative to interp1
dfEst(1) = lo + ((daPointer - floor(daPointer)) * (hi - lo));
if(dfEst(1) < 0.0)
   dfEst(1) = 0.0;
end
if(dfEst(1) > 260.0)
    dfEst(1) = 260.0;
end

% and then repeat as required
if (iterations > 1)
   dfPointerMult = 99/290;
   for m = 1:(iterations-1)
        % use improved df estimate to improve da estimate
        dfPointer = 1 + (dfEst(m) * dfPointerMult);
        if (dfPointer < 1)
            dfPointer = 1;
        end
        if (dfPointer > 100)
            dfPointer = 100;
        end
        xMax = max(xA(:,ceil(dfPointer)));
        xMin = min(xA(:,ceil(dfPointer)));
        temp = daMeasure;
        if (xMax < temp)
           temp = xMax;
        end
        if (temp < xMin)
           temp = xMin;
        end
        hi = interp1DLin(xA(:,ceil(dfPointer)),yA,temp); % faster alternative to interp1
        xMax = max(xA(:,floor(dfPointer)));
        xMin = min(xA(:,floor(dfPointer)));
        temp = daMeasure;
        if (xMax < temp)
            temp = xMax;
        end
        if (temp < xMin)
            temp = xMin;
        end
        lo = interp1DLin(xA(:,floor(dfPointer)),yA,temp); % faster alternative to interp1
        daEst(m+1) = lo + ((dfPointer - floor(dfPointer)) * (hi - lo));
        if(daEst(m+1) < 0.0)
            daEst(m+1) = 0.0;
        end
        if(daEst(m+1) > 96.0)
            daEst(m+1) = 96.0;
        end
        
        % now use improved da estimate to improve df estimate
        daPointer = 1 + (daEst(m+1) * daPointerMult);
        if (daPointer < 1)
            daPointer = 1;
        end
        if (daPointer > 100)
            daPointer = 100;
        end
        xMax = max(xF(ceil(daPointer),:));
        xMin = min(xF(ceil(daPointer),:));
        temp = dfMeasure;
        if (xMax < temp)
            temp = xMax;
        end
        if (temp < xMin)
            temp = xMin;
        end
        hi = interp1DLin(xF(ceil(daPointer),:),yF,temp); % faster alternative to interp1
        xMax = max(xF(floor(daPointer),:));
        xMin = min(xF(floor(daPointer),:));
        temp = dfMeasure;
        if (xMax < temp)
            temp = xMax;
        end
        if (temp < xMin)
            temp = xMin;
        end
        lo = interp1DLin(xF(floor(daPointer),:),yF,temp); % faster alternative to interp1
        dfEst(m+1) = lo + ((daPointer - floor(daPointer)) * (hi - lo));
        if(dfEst(m+1) < 0.0)
            dfEst(m+1) = 0.0;
        end
        if(dfEst(m+1) > 260.0)
            dfEst(m+1) = 260.0;
        end
    end
end

% take mean of daEst and dfEst 
temp = mean(dfEst(m:m+1));
clear dfEst;
dfEst = temp;
temp = mean(daEst(m:m+1));
clear daEst;
daEst = temp;
    
% correct amplitude
ampFactor = interp2DLin(daEst*daMult,dfEst*dfMult,ampCorrect);
aEst = ampFactor*mag(idx)*en;
    
% then correct according to window function
aEst = correctAmp(aEst,fEst,FFTSize,frameSize,ampFactor);   
    
% correct frequency (must be done after amp correction)
fEst = actualFreqMean(fEst,dfEst,daEst);

% correct phase using table look-up (setting dfEst to 0)
phiShift = interp2DLin(daEst*daMult,0,phiExpect);
phi=phi-phiShift;   % no need for +pi/2 as we doing inverse transform

% restore polarity to da and df estimates
dfEst = dfEst*dfSign;
daEst = daEst*daSign;
    




